rm(list=ls())

library(parallel)
library(x13binary)
library(seasonal)


setwd("/YOUR/WORKING/DIRECTORY")

source("func_RSVD_R_BREAKS.R")
source("func_RSVD_SA_SIM_NS.R")
source("func_RSVD_SA_SIM_NS_BREAK.R")

#=========== READ THE DATA =============

retail <- read.table(file="data.retail")$V2

obs <- length(retail)
my_season_period <- 12
num_period <- obs/my_season_period
time_start <- c(1992, 1)

ts_year_markers <- ts(seq(1992,2012), start = 1992, frequency = 1)

ts_retail <- ts(data = retail, start = time_start, frequency = my_season_period)

original_series <- log(ts_retail)

#=========== X13 - ARIMA =============

model_x13arima <- seas(log(ts_retail), 
                       regression.aictest = NULL, 
                       outlier = NULL, 
                       transform.function = "none", 
                       x11 = "")
static(model_x13arima)
summary(model_x13arima)

retail_sa_x13<-final(model_x13arima)
retail_seas_x13 <- series(model_x13arima,"x11.seasonal")

#=========== SEATS =============

model_seats <- seas(log(ts_retail), 
                    regression.aictest = NULL, 
                    outlier = NULL, 
                    transform.function = "none")
static(model_seats)
summary(model_seats)

retail_sa_seats<-final(model_seats)
retail_seas_seats <- series(model_seats,"seats.seasonal")

#=========== RSVD WITH NO BREAK =============

model_rsvd <- func_RSVD_SA_SIM_NS(dgp_y = log(ts_retail),season_period = my_season_period, max_r = 12)

retail_seas_rsvd <- ts(model_rsvd$season_svd, start = time_start, frequency = my_season_period)
retail_sa_rsvd <- log(ts_retail) - retail_seas_rsvd

#=========== RSVD WITH ABRUPT BREAKS =============

model_rsvdb <- func_RSVD_SA_SIM_NS_BREAK(dgp_y = log(ts_retail),season_period = my_season_period, max_r = 3)

retail_seas_rsvdb <- ts(model_rsvdb$season_svd, start = time_start, frequency = my_season_period)
retail_sa_rsvdb <- log(ts_retail) - retail_seas_rsvdb

#=========== PLOT THE RESULTS =============

plot(retail_seas_x13,col="red")
lines(retail_seas_seats,col="green")
lines(retail_seas_rsvd,col="blue")
lines(retail_seas_rsvdb,col="yellow")

plot(log(ts_retail))
lines(retail_sa_x13,col="red")
lines(retail_sa_seats,col="green")
lines(retail_sa_rsvd,col="blue")
lines(retail_sa_rsvdb,col="yellow")


