func_RSVD_R_BREAKS <- function(r_breaks, dgp_y, season_period) {
  
  #season_period = 12 #####
  #r_breaks = c(0,25)
  
  num_r <- length(r_breaks)
  num_period <- length(dgp_y)/season_period
  
  #==========================
  # BEGINNING OF RSVD METHOD
  #==========================
  # STEP 0: DEFINITIONS AND FUNCTIONS
  #==========================
  
  mat_Y <- matrix(dgp_y, nrow=num_period,ncol=season_period,byrow=TRUE)
  
  mat_Delta <- matrix(0, nrow=length(dgp_y)-1,ncol=length(dgp_y))
  for (i in 1:nrow(mat_Delta)) mat_Delta[i,i:(i+1)] <- c(-1,1)
  
  mat_Delta_sq <- t(mat_Delta) %*% mat_Delta
  
  mat_Q_n <- diag(1,num_period) - matrix(1/num_period,nrow = num_period,ncol=num_period)
  
  func_RSVD_BREAK <- function(one_break, mat_X){
    
    mat_eye <- diag(1,nrow(mat_X))
    
    mat_D <- matrix(0, nrow=nrow(mat_X)-2,ncol=nrow(mat_X))
    for (i in 1:nrow(mat_D)) mat_D[i, i:(i+2)] <- c(1,-2,1)
    
    mat_Omega <- t(mat_D)%*% mat_D
    
    if ( one_break != 0 ) {
      mat_X1 <- mat_X[1:one_break,]
      mat_X2 <- mat_X[-(1:one_break),]
      
      mat_eye1 <- mat_eye[1:one_break,1:one_break]
      mat_eye2 <- mat_eye[-(1:one_break),-(1:one_break)]
      
      mat_Omega1<- mat_Omega[1:one_break,1:one_break]
      mat_Omega2<- mat_Omega[-(1:one_break),-(1:one_break)]
    }
    
    criter <- 999
    iter<- 0
    u_ini <- princomp(mat_X)$scores[,1]
    
    while (criter > 1e-3 & iter<100){
      
      v <- t(mat_X)%*%u_ini
      
      v <- v / sqrt(sum(v^2))
      
      alpha_star <- c(NA, NA)
      
      if (one_break != 0){
        
        gcv1 <-function(nu1){
          mat_M1 <- solve(mat_eye1+(10^nu1)*mat_Omega1)
          mean(sum(((mat_eye1 - mat_M1)%*%mat_X1%*%v)^2))/(1-mean(diag(mat_M1)))^2
        } 
        
        re_gcv1 <- optim(0, gcv1 , method = "Brent", lower = -6, upper = 6)
        
        alpha_star1 <- 10^(re_gcv1$par)
        u1 <- solve(mat_eye1 + alpha_star1* mat_Omega1)%*% mat_X1 %*% v
        
        gcv2 <-function(nu2){
          mat_M2 <- solve(mat_eye2+(10^nu2)*mat_Omega2)
          mean(sum(((mat_eye2-mat_M2)%*%mat_X2%*%v)^2))/(1-mean(diag(mat_M2)))^2
        } 
        
        re_gcv2 <- optim(0, gcv2 , method = "Brent", lower = -6, upper = 6)
        
        alpha_star2 <- 10^(re_gcv2$par)
        u2 <- solve(mat_eye2 + alpha_star2* mat_Omega2)%*% mat_X2 %*% v
        
        alpha_star <- c(alpha_star1,alpha_star2)
        u <- rbind(u1,u2)
        criter <- max(abs( u -u_ini))
        iter <- iter + 1
        u_ini <- u
      } else {
        gcv <-function(nu){
          mat_M <- solve(mat_eye + (10^nu)* mat_Omega)
          mean(sum(((mat_eye-mat_M)%*%mat_X%*%v)^2))/(1-mean(diag(mat_M)))^2
        }  
        re_gcv <- optim(0, gcv , method = "Brent", lower = -6, upper = 6)
        
        alpha_star0 <- 10^(re_gcv$par)
        u <- solve(mat_eye + alpha_star0* mat_Omega)%*% mat_X %*% v
        
        alpha_star[1] <- alpha_star0
        criter <- max(abs(u-u_ini))
        iter <- iter + 1
        u_ini <- u
      }
    }
    
    output<-list(u_hat = as.vector(u), v_hat = as.vector(v))
  }
  
  #==========================
  #  BEGIN OF RSVD METHOD
  #==========================
  
  mat_YD <- matrix(0,ncol = season_period-1, nrow=num_period)
  for (i in 2:season_period) mat_YD[,i-1] <-mat_Y[,i] - mat_Y[,i-1]
  mat_Y_work <- mat_Q_n%*%mat_YD
  mat_Y_remain <- mat_Y_work
  
  #==========================
  # STEP 1: ESTIMATING SEASONAL COEFFICIENTS u FOR FIXED BREAKS
  #==========================
  
  mat_u_hat1 <- NULL
  
  for (iter in 1:num_r){
    
    temp_rsvd <- func_RSVD_BREAK(r_breaks[iter], mat_Y_remain)
    mat_u_hat1 <- cbind(mat_u_hat1,temp_rsvd$u_hat)
    if (iter!=num_r) mat_Y_remain <- mat_Y_remain - cbind(temp_rsvd$u_hat)%*%rbind(temp_rsvd$v_hat)
    
  }
  
  #==========================
  # STEP 2: ESTIMATING FIXED/TIME-VARYING SEASONALITY PATTERNS f AND v FOR FIXED BREAKS
  #==========================  
  
  mat_Z <- cbind( matrix(1,nrow=num_period) %x% diag(1,season_period), mat_u_hat1 %x% diag(1,season_period) )
  beta_hat <- MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%(t(mat_Z)%*%mat_Delta_sq%*%matrix(dgp_y,ncol=1) )
  mat_R <- diag( num_r+1 ) %x% matrix(1,nrow=1,ncol=season_period)
  vect_q <- matrix(0,ncol=1,nrow=num_r+1)
  beta_tilde <- beta_hat-MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%t(mat_R)%*%MASS::ginv( mat_R%*%MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%t(mat_R))%*%( mat_R%*%beta_hat-vect_q)
  
  mat_v_fixed <- cbind(beta_tilde[1:season_period])
  mat_fixed_hat <-  cbind(rep(1,num_period)) %x% t(mat_v_fixed)
  mat_v_hat2 <- matrix(beta_tilde[-(1:season_period)],nrow=season_period)
  
  #==========================
  # STEP 3: FINAL ESTIMATION OF SEASONAL COMPONENT FOR FIXED BREAKS
  #==========================
  
  matrix_season_hat <- mat_fixed_hat + mat_u_hat1 %*%t(mat_v_hat2)
  season_svd <- as.vector(t(matrix_season_hat))
  
  eval_breaks <- log(mean((diff(dgp_y-season_svd))^2)) + num_r * log(num_period)/num_period # sd(diff(dgp_y-season_svd))
  
  info_cri <- log(mean((diff(dgp_y-season_svd))^2)) + num_r * log(num_period)/num_period
  
  output <- list(mat_u_hat = mat_u_hat1, mat_v_fixed = mat_v_fixed, mat_v_hat = mat_v_hat2,
                 eval_breaks = eval_breaks, r_breaks = r_breaks, info_cri = info_cri,
                 season_svd = season_svd, final_r = length(r_breaks))

}


