func_RSVD_SA_SIM_NS <- function(dgp_y, season_period, max_r) {
  
  #==========================
  # DEFINITIONS AND FUNCTIONS
  #==========================
  
  func_RSVD <- function(mat_X){ 
    
    criter <- 999
    iter<- 0
    u_ini <- princomp(mat_X)$scores[,1]
    
    while (criter > 1e-3 & iter<100){
      
      v <- t(mat_X)%*%u_ini
      v <- v / sqrt(sum(v^2))
      
      gcv <-function(nu)  mean(sum(((diag(1,nrow(mat_X)) - solve(diag(1,nrow(mat_X)) + (10^nu)* mat_Omega)) %*% mat_X %*% v)^2)) / (1- mean(diag(solve(diag(1,nrow(mat_X)) + (10^nu)* mat_Omega))))^2
      re_gcv <- optim(0, gcv , method = "Brent", lower = -6, upper = 6)
      
      alpha_star <- 10^(re_gcv$par)
      u <- solve(diag(1,nrow(mat_X)) + alpha_star* mat_Omega)%*% mat_X %*% v
      criter <- mean((u-u_ini)^2)
      iter <- iter + 1
      u_ini <- u
    }
    output<-list(u_hat=as.vector(u),v_hat=as.vector(v),alpha_star=alpha_star)
  }
  
  func_EVAL <- function(mat_u_hat1){
    
    mat_Z <- cbind( matrix(1,nrow=num_period) %x% diag(1,season_period), mat_u_hat1 %x% diag(1,season_period) )
    beta_hat <- MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%(t(mat_Z)%*%mat_Delta_sq%*%matrix(dgp_y,ncol=1) )
    mat_R <- diag( num_r+1 ) %x% matrix(1,nrow=1,ncol=season_period)
    vect_q <- matrix(0,ncol=1,nrow=num_r+1)
    beta_tilde <- beta_hat-MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%t(mat_R)%*%MASS::ginv( mat_R%*%MASS::ginv(t(mat_Z)%*%mat_Delta_sq%*%mat_Z)%*%t(mat_R))%*%( mat_R%*%beta_hat-vect_q)
    
    mat_v_fixed <- cbind(beta_tilde[1:season_period])
    mat_fixed_hat <-  cbind(rep(1,num_period)) %x% t(mat_v_fixed)
    mat_v_hat2 <- matrix(beta_tilde[-(1:season_period)],nrow=season_period)
    
    matrix_season_hat <- mat_fixed_hat + mat_u_hat1 %*%t(mat_v_hat2)
    season_svd <- as.vector(t(matrix_season_hat))
    
    info_cri <- log(mean((diff(dgp_y-season_svd))^2))  + num_r * log(num_period)/num_period
    
    output <- list(info_cri = info_cri, 
                   mat_u_hat = mat_u_hat1, mat_v_fixed = mat_v_fixed, 
                   mat_v_hat = mat_v_hat2, season_svd = season_svd)
    
  }
  
  #=====================================================
  # SEARCHING FOR THE BEST NUMBER OF SEASONAL PATTERNS R
  #=====================================================
  
  num_period <- length(dgp_y)/season_period
  mat_Y <- matrix(dgp_y, nrow=num_period,ncol=season_period, byrow=TRUE)
  
  mat_D <- matrix(0, nrow=num_period-2,ncol=num_period)
  for (i in 1:nrow(mat_D))  mat_D[i, i:(i+2)] <- c(1,-2,1)
  mat_Omega <- t(mat_D)%*% mat_D
  
  mat_Delta <- matrix(0, nrow=length(dgp_y)-1,ncol=length(dgp_y))
  for (i in 1:nrow(mat_Delta)) mat_Delta[i,i:(i+1)] <- c(-1,1)
  mat_Delta_sq <- t(mat_Delta)%*% mat_Delta
  mat_Q_n <- diag(1,num_period) - matrix(1/num_period,nrow = num_period,ncol=num_period)
  
  mat_YD <- matrix(0,ncol = season_period-1, nrow=num_period)
  for (i in 2:season_period) mat_YD[,i-1] <-mat_Y[,i] - mat_Y[,i-1]
  mat_Y_remain <- mat_Q_n%*%mat_YD
  
  mat_u_hat1 <- NULL
  IC_old <- Inf
  results_candidates_last <- NULL
  model_results_last <- NULL
  
  for (num_r in 1:max_r){
    
    temp_rsvd <- func_RSVD(mat_Y_remain)
    
    mat_u_hat1 <-cbind(mat_u_hat1,temp_rsvd$u_hat)
    
    model_results_current <- func_EVAL(mat_u_hat1)
    
    IC_new <- model_results_current$info_cri
    
    if (IC_new > IC_old) {
      final_r <- num_r-1
      model_results_final <- model_results_last
      break 
    } else {
      if (num_r < max_r){
        IC_old <- IC_new 
        mat_u_hat1 <- model_results_current$mat_u_hat
        mat_Y_remain <- mat_Y_remain - cbind(temp_rsvd$u_hat)%*%rbind(temp_rsvd$v_hat)
        model_results_last <- model_results_current
      } else {
        final_r <- num_r
        model_results_final <- model_results_current
      }
    }
    
  }
  
  output <- model_results_final
  output$final_r <- final_r
  
  output
  
  
}




